//
//  HttpClient.m
//  YConnectSDKforIOSLibrary
//
//  Copyright (c) 2012年 Yahoo Japan Corporation. All rights reserved.
//

#import "HttpClient.h"

static BOOL checkSSL = true; //default true

/**
 * 証明書の検証
 * リリース時には削除(必須!!)
 **/
@implementation NSURLRequest(SSL)
/**
 * 証明書の検証の実行
 * @param NSString *host
 * @return YESを返す場合は証明書を無視、NOは証明書を確認する
 **/
+ (BOOL)allowsAnyHTTPSCertificateForHost:(NSString *)host {
    if (checkSSL) {
        return NO;
    } else {
        return YES;
    }
}
@end

@interface HttpClient()
- (void)requestCommon:(NSString *)urlString parameters:(HttpParameters *)parameters requestHeaders:(HttpHeaders *)requestHeaders httpMethod:(NSString *)httpMethod;
@end

/** 
 * Http通信を行うクラス
 **/
@implementation HttpClient

/**
 * 初期化
 **/
- (id)init {
    self = [super init];
    _responseCode = 0;
    _responseBody = @"";
    _responseHeaders = [[HttpHeaders alloc] init];
    return self;
}

/**
 * GETによる通信
 * @param NSString *urlString
 * @param HttpParameters *parameters
 * @param HttpHeaders *requestHeaders
 **/
- (void)requestGet:(NSString *)urlString parameters:(HttpParameters *)parameters requestHeaders:(HttpHeaders *)requestHeaders {

    //URLの作成
    //GETパラメータを整形して取得
    NSString *queryString = [parameters toQueryString];
    if (queryString != nil) {
        urlString = [urlString stringByAppendingFormat:@"?%@", queryString];
    }
    
    NSLog(@"%@: URL:%@", NSStringFromClass([self class]), urlString);
    
    if ([urlString hasPrefix:@"https"]) {
        NSLog(@"%@: HTTPS", NSStringFromClass([self class]));
        NSLog(@"%@: SSLCheck = %@", NSStringFromClass([self class]), checkSSL==YES? @"YES": @"NO");
    } else {
        NSLog(@"%@: HTTP", NSStringFromClass([self class]));
    }
    
    //URLリクエスト作成
    NSURL *url = [NSURL URLWithString:urlString];
    NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:url];
    [request setHTTPMethod:@"GET"];
    
    //リクエストヘッダ設定
    NSString *value;
    for (NSString *key in [requestHeaders allKeys]) {
        value = [requestHeaders objectForKey:key];
        [request addValue:value forHTTPHeaderField:key];
    }
    NSLog(@"%@: Headers: %@", NSStringFromClass([self class]), requestHeaders.toHeaderString);
    
    //リクエストに本文のセット
    if (self.requestBody != nil) {
        if ([self.requestBody isKindOfClass:[NSData class]]) {
            [request setHTTPBody:self.requestBody];
            NSLog(@"%@: Body: %@", NSStringFromClass([self class]), self.requestBody);
        } else if ([self.requestBody isKindOfClass:[NSString class]]) {
            [request setHTTPBody:[self.requestBody dataUsingEncoding:NSUTF8StringEncoding]];
            NSLog(@"%@: Body: %@", NSStringFromClass([self class]), self.requestBody);
        }
    }
    
    //タイムアウトのセット
    if (self.timeout != 0) {
        [request setTimeoutInterval:self.timeout];
    }
    self.timeout = [request timeoutInterval];
    NSLog(@"%@: Timeout: %f", NSStringFromClass([self class]), [request timeoutInterval]);
    
    //同期通信
    NSURLResponse *response = nil;
	NSError *error = nil;
    self.responseData = [
                         NSURLConnection
                         sendSynchronousRequest : request
                         returningResponse : &response
                         error : &error
                         ];
    
    self.response = response;
    self.error = error;
    [self readResponseBody];
    
}

/**
 * POSTによる通信
 * @param NSString *urlString
 * @param HttpParameters *parameters
 * @param HttpHeaders *requestHeaders
 **/
- (void)requestPost:(NSString *)urlString parameters:(HttpParameters *)parameters requestHeaders:(HttpHeaders *)requestHeaders {
    [self requestCommon:urlString parameters:parameters requestHeaders:requestHeaders httpMethod:@"POST"];
}

/**
 * PUTによる通信
 * @param NSString *urlString
 * @param HttpParameters *parameters
 * @param HttpHeaders *requestHeaders
 **/
- (void)requestPut:(NSString *)urlString parameters:(HttpParameters *)parameters requestHeaders:(HttpHeaders *)requestHeaders {
    [self requestCommon:urlString parameters:parameters requestHeaders:requestHeaders httpMethod:@"PUT"];
}

/**
 * DELETEによる通信
 * @param NSString *urlString
 * @param HttpParameters *parameters
 * @param HttpHeaders *requestHeaders
 **/
- (void)requestDelete:(NSString *)urlString parameters:(HttpParameters *)parameters requestHeaders:(HttpHeaders *)requestHeaders {
    [self requestCommon:urlString parameters:parameters requestHeaders:requestHeaders httpMethod:@"DELETE"];
}

/**
 * POST,PUT,DELETEの共通通信メソッド
 * @param NSString *urlString
 * @param HttpParameters *parameters
 * @param HttpHeaders *requestHeaders
 * @param NSString *httpMethod [Description]
 **/
- (void)requestCommon:(NSString *)urlString parameters:(HttpParameters *)parameters requestHeaders:(HttpHeaders *)requestHeaders httpMethod:(NSString *)httpMethod {
    
    NSLog(@"%@: URL:%@", NSStringFromClass([self class]), urlString);
    
    if ([urlString hasPrefix:@"https"]) {
        NSLog(@"%@: HTTPS", NSStringFromClass([self class]));
        NSLog(@"%@: SSLCheck = %@", NSStringFromClass([self class]), checkSSL==YES? @"YES": @"NO");
    } else {
        NSLog(@"%@: HTTP", NSStringFromClass([self class]));
    }
    
    //URLリクエスト作成
    NSURL *url = [NSURL URLWithString:urlString];
    NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:url];
    [request setHTTPMethod:httpMethod];
    
    //リクエストヘッダ設定
    NSString *value;
    for (NSString *key in [requestHeaders allKeys]) {
        value = [requestHeaders objectForKey:key];
        [request addValue:value forHTTPHeaderField:key];
    }
    NSLog(@"%@: Headers: %@", NSStringFromClass([self class]), requestHeaders.toHeaderString);
    
    //リクエストパラメータ設定
    [request setHTTPBody:[[parameters toQueryString] dataUsingEncoding:NSUTF8StringEncoding]];
    NSLog(@"%@: Parameters: %@", NSStringFromClass([self class]), [[NSString alloc] initWithData:[request HTTPBody] encoding:NSUTF8StringEncoding]);
    
    //リクエストに本文のセット
    if (self.requestBody != nil) {
        // *リクエストパラメータを上書きする
        if ([self.requestBody isKindOfClass:[NSData class]]) {
            [request setHTTPBody:self.requestBody];
            NSLog(@"%@: Body: %@", NSStringFromClass([self class]), self.requestBody);
        } else if ([self.requestBody isKindOfClass:[NSString class]]) {
            [request setHTTPBody:[self.requestBody dataUsingEncoding:NSUTF8StringEncoding]];
            NSLog(@"%@: Body: %@", NSStringFromClass([self class]), self.requestBody);
        }
    }

    //タイムアウトのセット
    //(iOS6以前だと、「POSTでかつリクエストのボディが空でない」場合は設定が無視され、240秒になってしまう)
    if (self.timeout != 0) {
        [request setTimeoutInterval:self.timeout];
    }
    self.timeout = [request timeoutInterval];
    NSLog(@"%@: Timeout: %f", NSStringFromClass([self class]), self.timeout);
    
    //同期通信
    NSURLResponse *response = nil;
	NSError *error = nil;
    self.responseData = [
                    NSURLConnection
                    sendSynchronousRequest : request
                    returningResponse : &response
                    error : &error
                    ];
    
    self.response = response;
    self.error = error;
    [self readResponseBody];
}
/**
 * レスポンス読み取り
 **/
- (void)readResponseBody {
    //ステータスコード取得
    self.responseCode = [(NSHTTPURLResponse *)self.response statusCode];
    NSLog(@"%@: Status Code[%d]", NSStringFromClass([self class]), self.responseCode);
    
    //レスポンスヘッダ取得
    NSDictionary *headers = [(NSHTTPURLResponse *)self.response allHeaderFields];
    NSString *value, *key;
    for (key in [headers allKeys]) {
        value = [headers objectForKey:key];
        [self.responseHeaders setValue:value forKey:key];
    }
    NSLog(@"%@: Response Headers:\n%@", NSStringFromClass([self class]), self.responseHeaders.toHeaderString);
    
    //レスポンス本文
    self.responseBody = [[NSString alloc]initWithData:self.responseData
                                         encoding:NSUTF8StringEncoding];
    NSLog(@"%@: Body:\n%@", NSStringFromClass([self class]), self.responseBody);
    
}

/**
 * Httpステータスコード取得
 * @return int
 **/
- (int)getStatusCode {
    return self.responseCode;
}
/**
 * 証明書検証を有効化
 **/
+ (void)enableSSLCheck {
    checkSSL = YES;
}
/**
 * 証明書検証を無効化
 **/
+ (void)disableSSLCheck {
    checkSSL = NO;
}

@end
