//
//  APIClient.m
//  YConnectSDKforIOSLibrary
//
//  Copyright (c) 2012年 Yahoo Japan Corporation. All rights reserved.
//

#import "APIClient.h"

static NSString * const POST_METHOD = @"POST";
static NSString * const GET_METHOD = @"GET";
static NSTimeInterval const DEFAULT_TIMEOUT = 60.0; // デフォルトタイムアウト

/**
 * APIにアクセスする
 **/
@implementation APIClient

/**
 * POST_METHODの静的取得
 * @return NSString * @"POST"
 **/
+ (NSString *)POST_METHOD {
    return POST_METHOD;
}
/**
 * GET_METHODの静的取得
 * @return NSString * @"GET"
 **/
+ (NSString *)GET_METHOD {
    return GET_METHOD;
}

/**
 * 初期化
 * @return id
 **/
- (id)init {
    self = [super init];
    _parameters = [[HttpParameters alloc] init];
    _requestHeaders = [[HttpHeaders alloc] init];
    _isShouldMoveAccessTokenFromParameterToHeader = true;
    _httpClient = [[HttpClient alloc] init];
    return self;
}
/**
 * アクセストークン(NSString)を用いた初期化
 * @param NSString *accessToken
 * @return id
 **/
- (id)initWithAccessTokenString:(NSString *)accessToken {
    self = [super init];
    _parameters = [[HttpParameters alloc] init];
    _requestHeaders = [[HttpHeaders alloc] init];
    [self setParameter:@"access_token" value:accessToken];
    _isShouldMoveAccessTokenFromParameterToHeader = true;
    _httpClient = [[HttpClient alloc] init];
    return self;
}
/**
 * アクセストークンを用いた初期化
 * @param BearerToken *accessToken
 * @return id
 **/
- (id)initWithAccessToken:(BearerToken *)accessToken {
    self = [super init];
    _parameters = [[HttpParameters alloc] init];
    _requestHeaders = [[HttpHeaders alloc] init];
    [self setParameter:@"access_token" value:accessToken.accessToken];
    _isShouldMoveAccessTokenFromParameterToHeader = true;
    _httpClient = [[HttpClient alloc] init];
    return self;
}
//URLとそのときのメソッドを指定して通信を行う
/**
 * URLとそのときのメソッドを指定して通信を行う
 * @param NSString *url
 * @param NSString *method @"GET",@"POST"(小文字でも可)
 * @throw APIClientException メソッド名がGETまたはPOSTではない例外
 * @throw APIClientException 接続失敗による例外
 **/
- (void)fetchResource:(NSString *)url method:(NSString *)method {
    
    //accessTokenをパラメータではなくヘッダに記述する場合
    if (self.isShouldMoveAccessTokenFromParameterToHeader) {
        [self moveAccessTokenFromParameterToHeader];
    }
    self.httpClient.timeout = self.apiTimeout; // タイムアウトセット
    
    //大文字小文字を無視したメソッドの判定
    if ([POST_METHOD caseInsensitiveCompare:method] == NSOrderedSame) {
        NSLog(@"POST");
        self.httpClient.requestBody = self.requestBody;
        [self.httpClient requestPost:url parameters:self.parameters requestHeaders:self.requestHeaders];
    } else if ([GET_METHOD caseInsensitiveCompare:method] == NSOrderedSame) {
        NSLog(@"GET");
        self.httpClient.requestBody = self.requestBody;
        [self.httpClient requestGet:url parameters:self.parameters requestHeaders:self.requestHeaders];
    } else {
        NSLog(@"%@: Undefined Http method", NSStringFromClass([self class]));
        //例外をスロー
        @throw [[APIClientException alloc]initWithName:@"MethodError" reason:@"Undefined Http method." userInfo:nil];
    }
    
    self.responseCode = self.httpClient.responseCode;
    self.responseBody = self.httpClient.responseBody;
    self.responseHeaders = self.httpClient.responseHeaders;
    
    [self checkErrorResponse];
}
/**
 * リクエストパラメータにセット
 * @param NSString *name パラメータ名(キー)
 * @param NSString *value
 **/
- (void)setParameter:(NSString *)name value:(NSString *)value {
    [self.parameters setValue:value forKey:name];
}
/**
 * リクエストヘッダにセット
 * @param NSString *field フィールド名(キー)
 * @param NSString *value
 **/
- (void)setHeader:(NSString *)field value:(NSString *)value {
    //コロンの除去
    field = [field stringByReplacingOccurrencesOfString:@":" withString:@""];
    //文字列の両端の空白を除去
    field = [field stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
    [self.requestHeaders setValue:value forKey:field];
}
/**
 * リクエストパラメータにアクセストークンをセット
 * @param accessToken
 **/
- (void)setAccessToken:(BearerToken *)accessToken {
    [self setParameter:@"access_token" value:accessToken.accessToken];
}

/**
 * accessTokenをリクエストパラメータからリクエストヘッダへ移動する
 **/
- (void)moveAccessTokenFromParameterToHeader{
    NSString *key = @"access_token";
    NSString *accessToken = [self.parameters objectForKey:key];
    if (accessToken != nil) {
        [self.requestHeaders setValue:[NSString stringWithFormat:@"Bearer %@", accessToken] forKey:@"Authorization"]; //ヘッダに追加
        [self.parameters.nsMutableDictionary removeObjectForKey:key]; //パラメータから削除
    }
}

- (void)checkErrorResponse{
    if (self.responseCode != 200) {
        if ([self.responseBody length] > 0) {
            NSError *error = nil;
            NSDictionary *jsonObject = [NSJSONSerialization
                                        JSONObjectWithData:[self.responseBody dataUsingEncoding:NSUTF8StringEncoding]
                                        options:NSJSONReadingAllowFragments
                                        error:&error];
            NSString *errorMessage = [jsonObject valueForKeyPath:@"Error.Message"];
            NSString *errorCode = nil;
            NSString *errorDescription = nil;
            if (errorMessage) {
                NSString *detail = [[errorMessage componentsSeparatedByString:@"Bearer"] objectAtIndex:1];
                NSArray *parameters = [detail componentsSeparatedByString:@","];
                NSMutableDictionary *paramDict = [NSMutableDictionary dictionary];
                for (int i = 0; i < [parameters count]; i++) {
                    NSArray *kv = [[parameters objectAtIndex:i] componentsSeparatedByString:@"="];
                    NSString *key = [[kv objectAtIndex:0] stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
                    NSString *value = [[[kv objectAtIndex:1] stringByReplacingOccurrencesOfString:@"\"" withString:@""] stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
                    [paramDict setValue:value forKey:key];
                }
                errorCode = [paramDict objectForKey:@"error"];
                errorDescription = [paramDict objectForKey:@"error_description"];
            } else {
                errorCode = [jsonObject objectForKey:@"error"];
                errorDescription = [jsonObject objectForKey:@"error_description"];
            }
            NSLog(@"%@: %@ / %@", NSStringFromClass([self class]), errorCode, errorDescription);
            @throw [[APIClientException alloc] initWithName:@"ResponseBodyContainsErrorMessage" reason:errorCode userInfo:nil];
        } else {
            NSLog(@"%@: An unexpected error has occurred.", NSStringFromClass([self class]));
            @throw [[APIClientException alloc] initWithName:@"" reason:@"An unexpected error has occurred." userInfo:nil];
        }
    }
}

@end


