//
//  AuthorizationCallBackUriParser.m
//  YConnectSDKforIOSLibrary
//
//  Copyright (c) 2012年 Yahoo Japan Corporation. All rights reserved.
//

#import "AuthorizationCallBackUriParser.h"

//Authorizational Code flowでのコールバック処理

@interface AuthorizationCallBackUriParser() 
- (void)parseUri:(NSURL *)responseUri;

@end

/**
 * Authorizational Code flowでのコールバック処理
 **/
@implementation AuthorizationCallBackUriParser

/**
 * 初期化
 * @param NSURL *redirectUri
 * @param NSString *callBackUri
 * @return id
 **/
- (id)initWithRedirectUri:(NSURL *)redirectUri callBackUri:(NSString *)callBackUri {
    self = [super init];
    parameters = [[HttpParameters alloc] init];
    
    _callBackUrl = callBackUri;
    [self parseUri:redirectUri];
    return self;
}
/**
 * コールバックされたURIの解析
 * @param NSURL *responseUri
 * @throw AuthorizationException コールバックURIが空であることによる例外
 * @throw AuthorizationException レスポンスボディにエラーが含まれることによる例外
 * @throw AuthorizationException Authパラメータが含まれていないことによる例外
 * @throw AuthorizationException コールバックURIが不正であることによる例外
 **/
- (void)parseUri:(NSURL *)responseUri {
    if (responseUri == nil) {
        NSLog(@"%@: Not Found Callback URI.", NSStringFromClass([self class]));
        //例外をスロー
        @throw [[AuthorizationException alloc] initWithName:@"NotFoundCallbackURI" reason:@"Not Found Callback URI." userInfo:nil];
    }
    NSLog(@"%@: Response Uri: %@", NSStringFromClass([self class]), responseUri);
    
    //parse query
    [self extractParameters:[responseUri query]];
    
    //parse fragment
    [self extractParameters:[responseUri fragment]];
    
    if ([[responseUri absoluteString] hasPrefix:self.callBackUrl]) {
        if ([parameters objectForKey:@"error"] != nil) {
            NSString *error, *errorDescription;
            NSDictionary *userInfo = nil;
            if ((error = [parameters objectForKey:@"error"]) == nil) {
                error = @"error";
            }
            if ((errorDescription = [parameters objectForKey:@"error_description"]) != nil) {
                userInfo = [NSDictionary dictionaryWithObject:errorDescription forKey:@"error_description"];
            }
            NSLog(@"%@: error=%@ error_description=%@", NSStringFromClass([self class]), error, errorDescription);
            //例外をスロー
            @throw [[AuthorizationException alloc] initWithName:@"ResponseBodyContainsErrorMessage" reason:error userInfo:userInfo];
        }
        if ([parameters count] == 0) {
            NSLog(@"%@: Not Found Authorization Parameters.", NSStringFromClass([self class]));
            //例外をスロー
            @throw [[AuthorizationException alloc] initWithName:@"NotFoundAuthorizationParameters" reason:@"Not Found Authorization Parameters." userInfo:nil];
        }
    } else {
        NSLog(@"%@: Invalid Callback URI.", NSStringFromClass([self class]));
        NSDictionary *userInfo = nil;
        if ([responseUri absoluteString] != nil) {
            userInfo = [NSDictionary dictionaryWithObject:[responseUri absoluteString] forKey:@"error_description"];
        }
        //例外をスロー
        @throw [[AuthorizationException alloc] initWithName:@"InvalidCallbackURI" reason:@"Invalid Callback URI." userInfo:userInfo];
    }
    NSLog(@"%@: Finished Parsing. %@", NSStringFromClass([self class]), parameters.toQueryString);
}
- (void)extractParameters:(NSString *)query {
    NSArray *keyValues = [query componentsSeparatedByString:@"&"];
    
    NSString *k, *v;
    for (NSString *keyValue in keyValues) {
        NSArray *kv = [keyValue componentsSeparatedByString:@"="];
        if ([kv count] == 2){
            k = [kv objectAtIndex:0];
            v = [kv objectAtIndex:1];
            [parameters setValue:v forKey:k];
            NSLog(@"%@: Put parameter. %@=%@", NSStringFromClass([self class]), k, v);
        }
    }
}
/**
 * ID_Tokenの取得
 * @return NSString *
 * @throw AuthorizationException IDトークンパラメータが含まれていないことによる例外
 **/
- (NSString *)getIdToken {
    NSString *idTokenString = [parameters objectForKey:@"id_token"];
    if (idTokenString != nil) {
        return idTokenString; 
    } else {
        NSLog(@"%@: Not found id_token parameter.", NSStringFromClass([self class]));
        //例外をスロー
        @throw [[AuthorizationException alloc] initWithName:@"NotFoundIdTokenPrameter" reason:@"Not found id_token parameter." userInfo:nil];
    }
}


@end
