//
//  CheckIdClient.m
//  YConnectSDKforIOSLibrary
//
//  Copyright (c) 2012年 Yahoo Japan Corporation. All rights reserved.
//

#import "CheckIdClient.h"

@implementation CheckIdClient

static NSTimeInterval const DEFAULT_TIMEOUT = 60.0; // デフォルトタイムアウト

/**
 * 初期化
 * @param NSString *endpointUrl
 * @param NSString *idToken
 * @return id
 **/
- (id)initWithEndpointUrl:(NSString *)endpointUrl idToken:(NSString *)idToken {
    self = [super init];
    _endpointUrl = endpointUrl;
    _idToken = idToken;
    _httpClient = [[HttpClient alloc] init];
    _checkIdTimeout = DEFAULT_TIMEOUT;
    return self;
}
/**
 * IDトークンの内容をチェックエンドポイントから取得
 * @throw CheckIdException 接続失敗による例外
 * @throw CheckIdException WWW-Authenticateヘッダにエラーメッセージが含まれていることによる例外
 * @throw CheckIdException HTTPステータスが200以外による例外
 **/
- (void)fetch {
    HttpParameters *parameters = [[HttpParameters alloc] init];
    [parameters setValue:self.idToken forKey:@"id_token"];
    self.httpClient.timeout = self.checkIdTimeout; // タイムアウトセット
    [self.httpClient requestPost:self.endpointUrl parameters:parameters requestHeaders:nil];
    
    //接続エラー
    if (self.httpClient.error != nil) {
        @throw [[CheckIdException alloc]initWithName:@"ConnectionError" reason:self.httpClient.error.localizedDescription userInfo:self.httpClient.error.userInfo];
    }
    
    NSDictionary *headers = self.httpClient.responseHeaders.nsMutableDictionary;
    NSString *wwwAuthHeader = [headers objectForKey:@"WWW-Authenticate"];
    if ([self.httpClient getStatusCode] != 200) {
        if (wwwAuthHeader != nil) {
            NSDictionary *headerDictionary = [CheckIdClient extractParameters:wwwAuthHeader];
            NSString *error, *errorDescription;
            NSDictionary *userInfo = nil;
            if ((error = [headerDictionary objectForKey:@"error"]) == nil) {
                error = @"error";
            }
            if ((errorDescription = [headerDictionary objectForKey:@"error_description"]) != nil) {
                userInfo = [NSDictionary dictionaryWithObject:errorDescription forKey:@"error_description"];
            }
            //例外スロー
            @throw [[CheckIdException alloc] initWithName:@"WWWAuthHeaderContainsErrorMessage" reason:error userInfo:userInfo];
        } else {
            //例外スロー
            @throw [[CheckIdException alloc] initWithName:@"FailedRequest" reason:@"Failed Request." userInfo:nil];
        }
    }
    
    NSString *jsonString = self.httpClient.responseBody;
    NSError *error = nil;
    NSDictionary *jsonObject = [NSJSONSerialization
                                JSONObjectWithData:[jsonString dataUsingEncoding:NSUTF8StringEncoding]
                                options:NSJSONReadingAllowFragments
                                error:&error];
    NSString *iss = [jsonObject objectForKey:@"iss"];
    NSString *userId = [jsonObject objectForKey:@"user_id"];
    NSString *aud = [jsonObject objectForKey:@"aud"];
    NSString *nonce = [jsonObject objectForKey:@"nonce"];
    long exp = [[jsonObject objectForKey:@"exp"] longValue];
    long iat = [[jsonObject objectForKey:@"iat"] longValue];
    self.idTokenObject = [[IdTokenObject alloc] initWithIss:iss userId:userId aud:aud nonce:nonce exp:exp iat:iat];
    
    NSString *dateHeader = [headers objectForKey:@"Date"];
    if (dateHeader != nil) {
        NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
        [dateFormatter setTimeStyle:NSDateFormatterFullStyle];
        [dateFormatter setDateFormat:@"EEE',' dd' 'MMM' 'yyyy HH':'mm':'ss zzz"];
        [dateFormatter setLocale:[[NSLocale alloc] initWithLocaleIdentifier:@"en_US"]];
        NSDate* date = [dateFormatter dateFromString:dateHeader];
        if (date) {
            self.currentTime = [date timeIntervalSince1970];
        } else {
            NSLog(@"could not parse date header: %@", dateHeader);
            self.currentTime = (long)floor([[NSDate date] timeIntervalSince1970]);
        }
    } else {
        self.currentTime = (long)floor([[NSDate date] timeIntervalSince1970]);
    }
    
}
/**
 * パラメータ抽出
 * @param NSString *wwwAuthHeader
 * @return NSDictionary *
 **/
+ (NSDictionary *)extractParameters:(NSString *)wwwAuthHeader {
    /// wwwAuthHeaderの例
    /// @"Bearer realm=\"yahooapis.jp\", error=\"insufficient_scope\", error_description=\"The access token expired\""
    NSMutableDictionary *dictionary = [[NSMutableDictionary alloc] init];
    NSArray *parameterArray = [wwwAuthHeader componentsSeparatedByString:@","];
    for (NSString *parameter in parameterArray) {
        //ダブルクォートの除去
        NSString *param = [parameter stringByReplacingOccurrencesOfString:@"\"" withString:@""];
        //KeyとValueに切り分け
        NSArray *keyValue = [param componentsSeparatedByString:@"="];
        if ([keyValue count] == 2) {
            NSString *key = [keyValue objectAtIndex:0];
            while ([key hasPrefix:@" "] && [key length] > 0) {
                key = [key substringFromIndex:1]; //先頭が空白なら、1文字除去して詰めていく
            }
            [dictionary setObject:[keyValue objectAtIndex:1]  forKey:key];
            NSLog(@"extractParameters: %@ / %@", [keyValue objectAtIndex:1], key);
        }
    }
    return dictionary;
}
@end
