//
//  IdTokenVerification.m
//  YConnectSDKforIOSLibrary
//
//  Copyright (c) 2012年 Yahoo Japan Corporation. All rights reserved.
//

#import "IdTokenVerification.h"

//10 minutes　発行後のIdToken検証ライブラリでの有効時間
static long const ACCEPTABLE_RANGE = 600L; //sec

@interface IdTokenVerification()
+ (long)ACCEPTABLE_RANGE;
@end

/**
 * IDトークンの内容を検証するクラス
 **/
@implementation IdTokenVerification

/**
 * IdToken検証の有効時間を取得
 * @return long
 **/
+ (long)ACCEPTABLE_RANGE {
    return ACCEPTABLE_RANGE;
}


/**
 * IDトークンの内容を検証
 * @param issuer 発行元
 * @param authNonce Authリクエスト時のNonce値
 * @param clientId クライアントID
 * @param iss IDトークンの発行元
 * @param aud IDトークンの発行対象
 * @param exp 有効期限
 * @param iat 発行時刻
 * @param nonce IDトークンのNonce値
 * @param currentTime 現在時刻
 * @throw IdTokenException 発行元が一致しないことによる例外
 * @throw IdTokenException Nonce値が一致しないことによる例外
 * @throw IdTokenException クライアントIDと発行対象が一致しないことによる例外
 * @throw IdTokenException トークン有効期限が有効期限外であることによる例外
 * @throw IdTokenException トークン発行時刻が許容範囲外であることによる例外
 **/
+ (void)check:(NSString *)issuer authNonce:(NSString *)authNonce clientId:(NSString *)clientId iss:(NSString *)iss aud:(NSString *)aud exp:(long)exp iat:(long)iat nonce:(NSString *)nonce currentTime:(long)currentTime {
    NSLog(@"%@:Check ID Token in the Claim from check id endpoint.", NSStringFromClass([self class]));
    
    //IdTokenの発行元と一致するか
    if (![issuer isEqualToString:iss]) {
        NSLog(@"%@:Invalid issuer.", NSStringFromClass([self class]));
        @throw [[IdTokenException alloc] initWithName:@"InvalidIssuer" reason:@"Invalid issuer." userInfo:nil];
    }
    //リプレイアタック防止用のパラメータが一致するか
    if (![authNonce isEqualToString:nonce]) {
        NSLog(@"%@:No match nonce.", NSStringFromClass([self class]));
        @throw [[IdTokenException alloc] initWithName:@"NoMatchNonce" reason:@"No match nonce." userInfo:nil];
    }
    //クライアントIDが発行対象のアプリと一致するか
    if (![clientId isEqualToString:aud]) {
        NSLog(@"%@:Invalid audience.", NSStringFromClass([self class]));
        @throw [[IdTokenException alloc] initWithName:@"InvalidAudience" reason:@"Invalid audience." userInfo:nil];
    }
    
    //現在時刻とトークン有効期限の比較(UNIXタイムスタンプ)
    NSLog(@"%@:Expiration: %ld(Current time: %ld)", NSStringFromClass([self class]), exp, currentTime);
    if (exp < currentTime) {
        NSLog(@"%@:Expired ID Token.", NSStringFromClass([self class]));
        @throw [[IdTokenException alloc] initWithName:@"ExpiredIdToken" reason:@"Expired ID Token." userInfo:nil];
    }
    
    //現在時刻とトークン発行時刻の差が許容範囲内か(UNIXタイムスタンプ)
    NSLog(@"%@:Current time - iat = %ld sec", NSStringFromClass([self class]), currentTime - iat);
    NSLog(@"%@:Issued time: %ld(Current time: %ld)", NSStringFromClass([self class]), iat, currentTime);
    if ((currentTime - iat) > ACCEPTABLE_RANGE) {
        NSLog(@"%@:Over acceptable range.", NSStringFromClass([self class]));
        @throw [[IdTokenException alloc] initWithName:@"OverAcceptableRange" reason:@"Over acceptable range." userInfo:nil];
    }
}

/**
 * 現在時刻を指定しないIDトークン検証のラッパー
 * @param issuer 発行元
 * @param authNonce Authリクエスト時のNonce値
 * @param clientId クライアントID
 * @param iss IDトークンの発行元
 * @param aud IDトークンの発行対象
 * @param exp 有効期限
 * @param iat 発行時刻
 * @param nonce IDトークンのNonce値
 **/
+ (void)check:(NSString *)issuer authNonce:(NSString *)authNonce clientId:(NSString *)clientId iss:(NSString *)iss aud:(NSString *)aud exp:(long)exp iat:(long)iat nonce:(NSString *)nonce {
    long currentTime = (long)floor([[NSDate date] timeIntervalSince1970]);
    [IdTokenVerification check:issuer authNonce:authNonce clientId:clientId iss:iss aud:aud exp:exp iat:iat nonce:nonce currentTime:currentTime];
}

/**
 * IdTokenObjectを指定したIDトークン検証のラッパー
 * @param issuer 発行元
 * @param authNonce Authリクエスト時のNonce値
 * @param clientId クライアントID
 * @param IdTokenObject *idTokenObject
 **/
+ (void)check:(NSString *)issuer authNonce:(NSString *)authNonce clientId:(NSString *)clientId idTokenObject:(IdTokenObject *)idTokenObject {
    NSString *iss = idTokenObject.iss;
    NSString *aud = idTokenObject.aud;
    long exp = idTokenObject.exp;
    long iat = idTokenObject.iat;
    NSString *nonce = idTokenObject.nonce;
    long currentTime = (long)floor([[NSDate date] timeIntervalSince1970]);
    [IdTokenVerification check:issuer authNonce:authNonce clientId:clientId iss:iss aud:aud exp:exp iat:iat nonce:nonce currentTime:currentTime];
}

/**
 * IdTokenObjectを指定したIDトークン検証のラッパー
 * @param issuer 発行元
 * @param authNonce Authリクエスト時のNonce値
 * @param clientId クライアントID
 * @param currentTime 現在時刻
 **/
+ (void)check:(NSString *)issuer authNonce:(NSString *)authNonce clientId:(NSString *)clientId idTokenObject:(IdTokenObject *)idTokenObject currentTime:(long)currentTime {
    NSString *iss = idTokenObject.iss;
    NSString *aud = idTokenObject.aud;
    long exp = idTokenObject.exp;
    long iat = idTokenObject.iat;
    NSString *nonce = idTokenObject.nonce;
    [IdTokenVerification check:issuer authNonce:authNonce clientId:clientId iss:iss aud:aud exp:exp iat:iat nonce:nonce currentTime:currentTime];
}

/**
 * IDトークンのJSON形式を指定したIDトークン検証のラッパー
 * @param issuer 発行元
 * @param authNonce Authリクエスト時のNonce値
 * @param clientId クライアントID
 * @param NSString *idTokenJSON
 **/
+ (void)check:(NSString *)issuer authNonce:(NSString *)authNonce clientId:(NSString *)clientId idTokenJSON:(NSString *)idTokenJSON {
    IdTokenObject *idTokenObject = [IdTokenVerification parseIdToken:idTokenJSON];
    NSString *iss = idTokenObject.iss;
    NSString *aud = idTokenObject.aud;
    long exp = idTokenObject.exp;
    long iat = idTokenObject.iat;
    NSString *nonce = idTokenObject.nonce;
    long currentTime = (long)floor([[NSDate date] timeIntervalSince1970]);
    [IdTokenVerification check:issuer authNonce:authNonce clientId:clientId iss:iss aud:aud exp:exp iat:iat nonce:nonce currentTime:currentTime];
}

/**
 * IDトークンのJSON形式を指定したIDトークン検証のラッパー
 * @param issuer 発行元
 * @param authNonce Authリクエスト時のNonce値
 * @param clientId クライアントID
 * @param NSString *idTokenJSON
 * @param currentTime 現在時刻
 **/
+ (void)check:(NSString *)issuer authNonce:(NSString *)authNonce clientId:(NSString *)clientId idTokenJSON:(NSString *)idTokenJSON currentTime:(long)currenTime {
    IdTokenObject *idTokenObject = [IdTokenVerification parseIdToken:idTokenJSON];
    NSString *iss = idTokenObject.iss;
    NSString *aud = idTokenObject.aud;
    long exp = idTokenObject.exp;
    long iat = idTokenObject.iat;
    NSString *nonce = idTokenObject.nonce;
    [IdTokenVerification check:issuer authNonce:authNonce clientId:clientId iss:iss aud:aud exp:exp iat:iat nonce:nonce currentTime:currenTime];
}

/**
 * JSON形式のIdTokenを解析してIdTokenObjectへ変換
 * @param NSString *json
 * @return IdTokenObject *
 **/
+ (IdTokenObject *)parseIdToken:(NSString *)json {
    IdTokenObject *idToken = [[IdTokenObject alloc] init];

    NSError *error = nil;
    NSDictionary *jsonObject = [NSJSONSerialization
                                JSONObjectWithData:[json dataUsingEncoding:NSUTF8StringEncoding]
                                options:NSJSONReadingAllowFragments
                                error:&error];
    
    idToken.iss = [jsonObject objectForKey:@"iss"];
    idToken.userId = [jsonObject objectForKey:@"user_id"];
    idToken.aud = [jsonObject objectForKey:@"aud"];
    idToken.nonce = [jsonObject objectForKey:@"nonce"];
    idToken.exp = [[jsonObject objectForKey:@"exp"] longValue];
    idToken.iat = [[jsonObject objectForKey:@"iat"] longValue];
    
    NSLog(@"%@:Parse ID Token and converted it to a IdTokenObject.", NSStringFromClass([self class]));
    return idToken;
}

@end
