//
//  YConnectExplicit.m
//  YConnectSDKforIOSLibrary
//
//  Copyright (c) 2012年 Yahoo Japan Corporation. All rights reserved.
//
#import <UIKit/UIKit.h>

#import "YConnectExplicit.h"
#import "ExplicitCallbackUriParser.h"
#import "TokenClient.h"
#import "RefreshTokenClient.h"


//リクエスト先
static NSString * const AUTHORIZATION_ENDPOINT_URL = @"https://auth.login.yahoo.co.jp/yconnect/v1/authorization";
static NSString * const TOKEN_ENDPOINT_URL = @"https://auth.login.yahoo.co.jp/yconnect/v1/token";
static NSString * const CHECK_ID_ENDPOINT_URL = @"https://auth.login.yahoo.co.jp/yconnect/v1/checktoken";
static NSString * const USERINFO_ENDPOINT_URL = @"https://userinfo.yahooapis.jp/yconnect/v1/attribute";
static NSString * const ISSUER = @"https://auth.login.yahoo.co.jp";

static YConnectExplicit *instance;

/**
 * Explicit flow によるYConnectの利用
 **/
@implementation YConnectExplicit

/**
 * 初期化
 * @return id
 **/
- (id)init {
    //デフォルト値
    self = [super init];
    if (self) {
        _requestClient = [[AuthorizationRequestClient alloc] init]; // クライアントIDなどをセットせずに単純に初期化
        _responseType = [OAuth2ResponseType CODE_IDTOKEN];
        _display = [OIDCDisplay SMART_PHONE];
        _prompt = [OIDCPrompt LOGIN];
        _scope = nil;
        _nonce = nil;
        _checkIdClient = [[CheckIdClient alloc] init];
        _parameterDictionary = [[NSMutableDictionary alloc] init];
        
    }
    return self;
}
/**
 * 初期化
 * @param NSString *clientId
 * @param NSString *redirectUri
 * @param NSString *state
 * @return id
 **/
- (id)initWithClientId:(NSString *)clientId redirectUri:(NSString *)redirectUri state:(NSString *)state {
    self = [super init];
    if (self) {
        _requestClient = [[AuthorizationRequestClient alloc] initWithEndpointUri:AUTHORIZATION_ENDPOINT_URL clientId:clientId];
        _requestClient.redirectUri = redirectUri;
        _requestClient.state = state;
        _checkIdClient = [[CheckIdClient alloc] init];
        _parameterDictionary = [[NSMutableDictionary alloc] init];
    }
    return self;
}
/**
 * 初期化
 * @param NSString *clientId
 * @param NSString *redirectUri
 * @param NSString *state
 * @param NSString *responseType
 * @param NSString *display
 * @param NSString *prompt
 * @param NSString *scope
 * @param NSString *nonce
 * @return id
 **/
- (id)initWithClientId:(NSString *)clientId redirectUri:(NSString *)redirectUri state:(NSString *)state responseType:(NSString *)responseType display:(NSString *)display prompt:(NSString *)prompt scope:(NSString *)scope nonce:(NSString *)nonce {
    self = [super init];
    if (self) {
        _requestClient = [[AuthorizationRequestClient alloc] initWithEndpointUri:AUTHORIZATION_ENDPOINT_URL clientId:clientId];
        _requestClient.redirectUri = redirectUri;
        _requestClient.state = state;
        _responseType = responseType;
        _display = display;
        _prompt = prompt;
        _scope = scope;
        _nonce = nonce;
        _checkIdClient = [[CheckIdClient alloc] init];
        _parameterDictionary = [[NSMutableDictionary alloc] init];
    }
    return self;
}

/**
 * インスタンス取得
 * @return YConnectImplicit *
 **/
+ (YConnectExplicit *)getInstance {
    if(instance == nil) {
        instance = [[YConnectExplicit alloc] init];
    }
    return instance;
}

/**
 * Authorization URI生成
 * @return NSURL *
 **/
- (NSURL *)generateAuthorizationUri {
    self.requestClient.responseType = self.responseType;
    [self.requestClient setParameter:@"display" value:self.display];
    [self.requestClient setParameter:@"prompt" value:self.prompt];
    if (self.scope != nil) {
        [self.requestClient setParameter:@"scope" value:self.scope];
    }
    if (self.nonce != nil) {
        [self.requestClient setParameter:@"nonce" value:self.nonce];
    }
    
    self.requestClient.dic = self.parameterDictionary;
    
    return [self.requestClient generateAuthorizationUri];
}

/**
 * Authorization URI生成
 * @param NSString *prompt
 * @return NSURL *
 **/
- (NSURL *)generateAuthorizationUriWithPrompt:(NSString *)prompt {
    self.requestClient.responseType = self.responseType;
    [self.requestClient setParameter:@"display" value:self.display];
    [self.requestClient setParameter:@"prompt" value:prompt];
    if (self.scope != nil) {
        [self.requestClient setParameter:@"scope" value:self.scope];
    }
    if (self.nonce != nil) {
        [self.requestClient setParameter:@"nonce" value:self.nonce];
    }
    
    self.requestClient.dic = self.parameterDictionary;
    
    return [self.requestClient generateAuthorizationUri];
}

/**
 * 同意画面を表示する
 **/
- (void)requestAuthorization {
    //NSURL *url = [NSURLURLWithString:@"https://www.apple.com/jp/"];
    NSURL *url = [self generateAuthorizationUri];
    [[UIApplication sharedApplication] openURL:url];
}

/**
 * Authorizationエンドポイントのレスポンス解析
 * @param NSURL *uri AuthからコールバックされてきたURL
 * @param NSString *customUriScheme
 * @param NSString *state Authにリクエストしたstate値
 **/
- (void)parseAuthorizationResponse:(NSURL *)uri customUriScheme:(NSString *)customUriScheme state:(NSString *)state {
    ExplicitCallbackUriParser *responseParser = [[ExplicitCallbackUriParser alloc] initWithRedirectUri:uri callBackUri:customUriScheme];
    self.code = [responseParser getAuthorizationCode:state];
}

/**
 * authorization code取得
 * @return NSString *
 **/
- (NSString *)getAuthorizationCode {
    return self.code;
}

/**
 * Tokenエンドポイントにリクエストする
 * @param NSString *code authorization code
 * @param NSString *redirectUri AuthからコールバックされてきたURL
 * @param NSString *clientId
 **/
- (void)requestToken:(NSString *)code redirectUri:(NSString *)redirectUri clientId:(NSString *)clientId {
    TokenClient *tokenClient = [[TokenClient alloc] initWithEndpointUrl:TOKEN_ENDPOINT_URL authorizationCode:code redirectUri:redirectUri clientId:clientId];
    [tokenClient fetch];
    self.accessToken = [tokenClient getAccessToken];
    self.idToken = [tokenClient getIdToken];
}

/**
 * アクセストークン取得
 * @return NSString *
 **/
- (NSString *)getAccessToken {
    return self.accessToken.accessToken;
}

/**
 * アクセストークンの有効期限取得
 * @return long
 **/
- (long)getAccessTokenExpiration {
    return self.accessToken.expiration;
}

/**
 * リフレッシュトークン取得
 * @return NSString *
 **/
- (NSString *)getRefreshToken {
    return self.accessToken.refreshToken;
}

/**
 * IDトークン取得
 * @return NSString *
 **/
- (NSString *)getIdToken {
    return self.idToken;
}

/**
 * IDトークンをチェックポイントに送り、IDトークンの内容を取得
 * @param NSString *idToken
 * @param NSString *nonce authorizationエンドポイントリクエスト時に指定したnonce
 * @param NSString *clientId
 **/
- (void)requestCheckIdToken:(NSString *)idToken nonce:(NSString *)nonce clientId:(NSString *)clientId {
    CheckIdClient *checkIdClient = [[CheckIdClient alloc] initWithEndpointUrl:CHECK_ID_ENDPOINT_URL idToken:idToken];
    //タイムアウトのセット
    if (self.checkIdClientTimeout != 0) {
        checkIdClient.checkIdTimeout = self.checkIdClientTimeout;
    }
    [checkIdClient fetch]; //IDトークンの内容を取得
    self.idTokenObject = checkIdClient.idTokenObject;
    [IdTokenVerification check:ISSUER authNonce:nonce clientId:clientId idTokenObject:self.idTokenObject currentTime:checkIdClient.currentTime];
}

/**
 * IDトークンをオブジェクトとして取得
 * @return id
 **/
- (id)getIdTokenObject {
    return self.idTokenObject;
}

/**
 * アクセストークンを更新
 * @param NSString *refreshToken
 * @param NSString *clientId
 **/
- (void)refreshAccessToken:(NSString *)refreshToken clientId:(NSString *)clientId {
    RefreshTokenClient *refreshTokenClient = [[RefreshTokenClient alloc] initWithEndpointUrl:TOKEN_ENDPOINT_URL refreshToken:refreshToken clientId:clientId];
    [refreshTokenClient fetch];
    self.accessToken = [refreshTokenClient getAccessToken];
}

- (void)requestUserInfo:(NSString *)accessTokenString {
    UserInfoClient *userInfoClient = [[UserInfoClient alloc] initWithAccessTokenString:accessTokenString];
    [userInfoClient fetchResource:USERINFO_ENDPOINT_URL method:[APIClient GET_METHOD]];
    self.userInfoObject = [userInfoClient getUserInfoObject];
}

- (id)getUserInfoObject {
    return self.userInfoObject;
}

/**
 * URLのパラメータに追加するその他の汎用的なパラメータと値をセット
 * @param NSString *parameter
 * @param NSString *value
 **/
- (void)addParameter:(NSString *)parameter value:(NSString *)value {
    [self.parameterDictionary setObject:value forKey:parameter];
}
@end
